package Caboodle;
#
#=============================================================================
# 2005/12/05 mne  : added transparancy parameter in the fillGraph subroutine
# 2006/04/26 eat  : updated version to 0.5.2
# 2006/08/07 eat  : dropping bad characters in text() function
# 2007/04/09 eat 0.6.1  Fixed bug in note delete link under Apache
#            Using $user_CertId in place of $ENV{CLIENT_CERT_SUBJECT_CN}
# 2007/11/18 eat 0.6.8  Extracted portions of EQL.pm use as a caboodle web framework
#=============================================================================

=head1 NAME

Caboodle - Caboodle Web API

=head1 SYNOPSIS

  use lib "../lib";
  use NBK::Caboodle;

=head1 DESCRIPTION

This module provides subroutines for common EQL database functions. 

=cut

use POSIX;

#========================================================================================
# Configuration Routines
#========================================================================================

# Load an EQL configuration file into a hash
#
#   $msg=Caboodle::loadConfig(\%hash,"<file>");
#
#   Example:  $msg=Caboodle::loadConfig(\%hash,"/opt/cipher/web/eql.cfg");
#
# You can specify any variable in the config file using the following
# syntax.
#
#   # <comment>
#   <variable> = "<value>"; [ # <comment> ]
#
# Here are some of the well known variables.
#
#   db        - database DBI connection (after open)
#   dbname    - database name (tns alias name)
#   dbuser    - database user name
#   dbpass    - database user password
#
#   path
#   cgi
#   nbq       - NodeBrain queue
#
# You reference these variables using the hash specified as the first
# argument.  If your hash is %eql (as in the example above), then you 
# would reference a configuration variable named 'dbname' as follows.
#
#   $eql{dbname}
#
# Returns '' if successful, otherwise returns an error message
#

sub loadConfig{
  (my $eql,my $cfg)=@_;
  
  open(CFG,$cfg)||return("Unable to open $cfg");
  my $i=0;    # line counter
  my $line;
  while(<CFG>){
    $i++;
    next if(/^\s*(#|$)/);
    $line=$_;
    if(/^\s*([a-zA-Z]+)\s*=\s*"(.*?)"\s*;/){$$eql{$1}=$2;}
    else{
      close(CFG);
      return("Syntax error in $cfg line $i: $line");
      }
    }
  close(CFG);
  return('');
  }

# print configuration hash (assumes simple string or number values)

sub printConfig{
  (my $eql)=@_;
  foreach $var (sort keys %$eql){
    print("$var=\"$$eql{$var}\"\n");
    }
  }

#======================================================================================================
# User Settings Functions
#
#   User settings are defined in a user's settings configuration file.
#
#     user/<userid>/settings.cfg
#
#   The file format is:
#
#     <variable> = '<value>'; # <comment>
#     # <comment>
#
#     Variable names must start with an alphabetic character [a-zA-Z] and may only contain
#     alphanumeric characters [a-zA-Z0-9].
#
#   Default settings are configured in the default user directory.
#
#     user/default/settings.cfg
#
#   Easy reference variables are included here to simplify the syntax for common settings.
#   These varaibles have defaults that may be overridden by the default settings.cfg and/or the
#   the user's settings.cfg
#
#======================================================================================================
                            # The cert variables are not user preferences, but are related
$user_CertId='';            # userId from certificate
$user_CertEmail='';         # user email address from certificate

%user;                      # user preference hash and easy reference variables
$user_Id='';                # userId (may be different than certId - using this allows for testing as another user)
$user_Email='';             # email address (may be different than certEmail)
$user_DateSep='/';          # yyyy/dd/mm
$user_TimeSep=':';          # hh:mm:ss
$user_DateTimeSep='_';      # yyyy/dd/mm_hh:mm:ss
$user_TimeZone=0;           # GMT
$user_StyleSheet='caboodle';# HTML style sheet (colors font etc.)

# Get information from the user's certificate and return the user identifier (BEMSID)

sub getCertUserId{
  return($ENV{SSL_CLIENT_S_DN_CN}) if($ENV{SSL_CLIENT_S_DN_CN}); # apache mod_ssl
  return($ENV{CLIENT_CERT_SUBJECT_CN}) if($ENV{CLIENT_CERT_SUBJECT_CN}); # Sun One iPlanet
  return('guest'); # Temp - while running without certificates
  }

sub getCertUserEmail{
  return($ENV{SSL_CLIENT_S_DN_Email}) if($ENV{SSL_CLIENT_S_DN_Email}); # apache mod_ssl
  return($ENV{CLIENT_CERT_SUBJECT_E}) if($ENV{CLIENT_CERT_SUBJECT_E});  # Sun One iPlanet
  return('root@localhost'); # Should use configured guest address instead
  }

# Load user settings from the user data store
#
#   This function may be called to load any user's settings into
#   a specified hash.  See setUser() for setting the user preferences
#   for a given EQL environment.

sub loadUserSettings{
  my ($userId,$userEnv)=@_;
  my $msg=loadConfig($userEnv,"user/$userId/settings.cfg");
  $$userEnv{Id}=$userId;       # user can not override in settings file
  }

#
# Set user settings to those of specified user or, by default, the authenticated user
#
#   We first load the default settings and then let the user's config override them
#
# Note: The quick reference variables and %user hash are global variables for this module,
#       not associated with a specific EQL environment.  The $$eql{UserSettings} variable
#       references environment specific user settings.  Where it seems appropriate
#       we should reference the settings via the environment specific reference.  The
#       global preferences are just a convenience---where we don't have an $eql passed
#       as a parameter.
#
sub setUser{
  my ($userId)=@_;               # Environment and optional user

  $user_CertId=getCertUserId();             # Make sure we know who the real user is
  $user_CertEmail=getCertUserEmail();
  $userId=$user_CertId if(!$userId);        # This enables us to test as a different user
  $user{Id}=$userId;                        # Load builtin defaults for easy reference variables
  $user{Email}=$user_CertEmail;
  $user{DateSep}=$user_DateSep;
  $user{TimeSep}=$user_TimeSep;
  $user{DateTimeSep}=$user_DateTimeSep;
  $user{TimeZone}=$user_TimeZone;
  $user{StyleSheet}=$user_StyleSheet;
  loadUserSettings('default',\%user);            # load default settings from config file
  loadUserSettings($userId,\%user);              # load user's settings from config file
  $user_Id=$user{Id};                       # update the easy reference variables
  $user_Email=$user{Email};
  $user_DateSep=$user{DateSep};
  $user_TimeSep=$user{RimeSep};
  $user_DateTimeSep=$user{DateTimeSep};
  $user_TimeZone=$user{TimeZone};
  $user_StyleSheet=$user{StyleSheet};
  $user{CertId}=$user_CertId;               # plug certificate varaibles for viewing only
  $user{CertEmail}=$user_CertEmail;
  return(\%user);
  }

#======================================================================================================
# Log functions
#
#   This is for logging events related to an EQL database and user interface
#======================================================================================================

sub log{
  (my $eql,my $message)=@_;
  (my $sec,my $min,my $hour,my $mday,my $mon,my $year,my $wday,my $yday)=gmtime(time());
  my $timestamp=sprintf("%4.4d/%2.2d/%2.2d_%2.2d:%2.2d:%2.2d",$year,$mon,$mday,$hour,$min,$sec);
  # the path should come from the eql configuration file
  #system("echo \"$timestamp $message\" >> log/eql.log\n");
  }

#======================================================================================================
# Display functions
#======================================================================================================

sub displayInsert{
  (my $eql,my $file)=@_;

  open(FILE,"$file");
  while(<FILE>){
    print;
    }
  close(FILE)
  }

# 2005/09/18 eat - included query to keep drilling down
# 2011-03-07 eat - replaced third parameter ($pivQuery) with $script
sub displayPageHead{
  my ($eql,$title,$script)=@_;

  print("<html>
<head>
<title>$title</title>
<link rel='shortcut icon' href='/nb.ico'>
<meta name='Content-Style' content='text/css'>
<link rel='stylesheet' title='caboodle_style' href='/style/$user_StyleSheet.css' type='text/css'>
<meta http-equiv='Default-Style' content='caboodle_style'>
$script
</head>
<body marginwidth='0' marginheight='0' leftmargin='0' topmargin='0'>\n");
  }

sub displayUserSettings{
  (my $eql)=@_;
  print("<b>User $user_Id Settings</b>\n");
  print("<table><tr><th>Variable</td><th>Value</th></tr>\n");
  foreach $var (sort keys %user){
    print("<tr><td>$var</td><td>\"$user{$var}\"</td></tr>\n");
    }
  print("</table>\n");
  print("<br><b>User $user_Id Settings Config File</b><pre>\n");
  open(FILE,"user/$user_Id/settings.cfg");
  while(<FILE>){
    print;
    }
  close(FILE);
  print("</pre>\n");
  print("<b>Default User Settings Config File</b><pre>\n");
  open(FILE,"user/default/settings.cfg");
  while(<FILE>){
    print;
    }
  close(FILE);
  print("</pre>\n");
  }

1
