#!/usr/bin/perl
# Copyright(c) 2006-2007 The Boeing Company.
#                        Ed Trettevik <eat@nodebrain.org>
#===============================================================================
# Package:   NodeBrain Planner
#
# File:      Control.pm
#
# Path:      CABOODLE/lib/NBK/Plan
#
# Title:     Control Element Class
#
# Function:
#
#   This is a Perl class module for the CONTROL element of the NodeBrain Planner
#   XML schema.  The CONTROL element is associated with PLAN schema 'Control'. 
#
# Synopsis:
#
#   use NBK::Planner::Control
# 
# XML Schema:
#
#   Parents: plan
#
#   <control scheme='<scheme>' schema='<schema>'>
#     <permits>
#       <permit user='<userId>' role='<role'>
#     </permits>
#   </control>
#
#===============================================================================
# Change History:
#
#   Date      Name/Change
# ----------  ------------------------------------------------------------------
# 2007/08/02  Ed Trettevik - created using code from original Plan.pm
#===============================================================================
package Control;
use Carp;
use NBK::Planner::Planner;
use NBK::Planner::Permits;
#use NBK::Planner::Actions;

#===============================================================================
# Planner Element Methods
#===============================================================================

# Create new object

sub new{
  my ($package,$planner,$path,$doc)=@_;
  bless {PLANNER => $planner,DOC => $doc, PATH => $path} => $package;
  }

# Generate XML representation of document
#   This is where we update and possibly upgrade the document

sub xml{
  my ($self,$path,$element,$action,$canEdit,$canCntl)=@_;
  #print("Control.xml called path=$path element=$element action=$action<br>\n");
  my $planner=$self->{PLANNER};
  my $doc=$self->{DOC};
  $planDoc=$doc->child('plan');
  $controlDoc=$planDoc->child('control');
  my $scheme=$controlDoc->attribute('scheme');
  $scheme=$controlDoc->attribute('type') if(!$scheme);
  $schema=$controlDoc->attribute('schema');
  my $childElement=$1 if($element=~/^Control.(.*)$/);
  my $childPath="$path:Control";
  my $xml="<control scheme='$scheme' schema='$schema'>\n";
  my $permitsDoc=$controlDoc->child('permits');
  if($permitsDoc){
    my $permitsObj=new Permits($planner,$permitsDoc);
    my $permitsXml=$permitsObj->xml($childPath,$childElement,$action,$canEdit,$canCntl);
    return($permitsXml) if(!($permitsXml=~/^</));
    $xml.=$permitsXml;
    }
  # drop actions
  #my $actionsDoc=$controlDoc->child('actions');
  #if($actionsDoc){
  #  my $actionsObj=new Actions($planner,$actionsDoc);
  #  my $actionsXml=$actionsObj->xml($childPath,$childElement,$action,$canEdit,$canCntl);
  #  return($actionsXml) if(!($actionsXml=~/^</));
  #  $xml.=$actionsXml;
  #  }
  
  # drop skulls
  #my $skullsDoc=$controlDoc->child('skulls');
  #if($skullsDoc){
  #  $xml.="<actions>\n";
  #  my $id=0;
  #  foreach my $skullDoc ($skullsDoc->children('skull')){
  #    my $skullName=$skullDoc->attribute('name');
  #    my $skullContext=$skullDoc->attribute('context');
  #    $id++;
  #    $xml.="<action id='$id' on='TableRowUpdate' do='TableRowUpdate $skullName $skullContext \"$\${detail}\"'/>\n";
  #    }
  #  $xml.="</actions>\n";
  #  }

  $xml.="</control>\n";
  return($xml);
  }
  
# Generate an html representation
#   This representation may vary depending on the user's permissions

sub html{
  my ($self,$path,$element,$action,$canEdit,$canCntl)=@_;
  #print("Control.html called with path=$path element=$element action=$action canEdit=$canEdit canCntl=$canCntl<br>\n");
  my $planner=$self->{PLANNER};
  my $doc=$self->{DOC};
  $planDoc=$doc->child('plan');
  $controlDoc=$planDoc->child('control');
  my $scheme=$controlDoc->attribute('scheme');
  $scheme=$controlDoc->attribute('type') if(!$scheme);
  #my $html="<table width='100%' cellpadding=0 cellspacing=0>\n<tr><th><center>Control</center></th></tr>\n<tr><td>\n";
  my $html="<table width='100%' cellpadding=0 cellspacing=0>\n<tr><td>\n";

  my $childElement=$1 if($element=~/^Control.(.*)$/);
  my $childPath="$path:Control";
  my $permitsDoc=$controlDoc->child('permits');
  if($permitsDoc){
    my $permitsObj=new Permits($planner,$permitsDoc);
    $html.=$permitsObj->html($childPath,$childElement,$action,$canEdit,$canCntl);
    }

  #my $actionsDoc=$controlDoc->child('actions');
  #if($actionsDoc){
  #  my $actionsObj=new Actions($planner,$actionsDoc);
  #  $html.=$actionsObj->html($childPath,$childElement,$action,$canEdit,$canCntl);
  #  }

  # SKULLS is deprecated and can be removed once plans are converted by an update (see xml{} method above)
  # Skull Table
  #my $skullsDoc=$controlDoc->child('skulls');
  #if($skullsDoc){
  #  $html.="<table><tr><th colspan=2>Skulls</th></tr><tr><th>Name</th><th>Context</th></tr>\n";
  #  foreach my $skullDoc ($skullsDoc->children('skull')){
  #    my $skullName=$skullDoc->attribute('name');
  #    my $skullContext=$skullDoc->attribute('context');
  #    $html.="<tr><td>$skullName</td><td>$skullContext</td></tr>\n";
  #    }
  #  $html.="</table>\n";
  #  }

  $html.="</td></tr></table>\n";
  return($html);
  }

# Compile into NodeBrain comments
#

sub nb{
  my ($self)=@_;
  my $planner=$self->{PLANNER};
  my $doc=$self->{DOC};
  my $planDoc=$doc->child('plan');

  my $nb='';

  if(!$planDoc){
    $nb.="# NBK000E Expecting PLAN element as root of document\n";
    return($nb);
    }

  my $controlDoc=$planDoc->child('control');
  if(!$controlDoc){
    $nb.="# NBK000E Expecting CONTROL element within PLAN element\n";
    return($nb);
    }
  my %controlAttrKnown=(scheme => 1, type => 1, schema => 1);
  my %controlAttr=$controlDoc->attributes;
  foreach my $controlAttr (keys %controlAttr){
    $nb,"# NBK000E CONTROL attribute '$controlAttr' is not recognized.\n" if(!$controlAttrKnown{$controlAttr});
    }
  my $controlScheme=$controlDoc->attribute('scheme');
  $controlScheme=$controlDoc->attribute('type') if(!$controlScheme);
  $nb.="# NBK000E Scheme attribute required in CONTROL element.\n" if(!$controlScheme);
  
  my $permitsDoc=$controlDoc->child('permits');
  if($permitsDoc){
    my $permitsObj=new Permits($planner,$permitsDoc);
    $nb.=$permitsObj->nb();
    }
  #my $actionsDoc=$controlDoc->child('actions');
  #if(!$actionsDoc){
  #  my $actionsObj=new Actions($planner,$actionsDoc);
  #  $nb.=$actionsObj->nb();
  #  }

  #my $skullsDoc=$controlDoc->child('skulls');
  #if($skullsDoc){
  #  $nb.="#\n# Skulls:\n#\n";
  #  foreach my $skullDoc ($skullsDoc->children('skull')){
  #    my $skullName=$skullDoc->attribute('name');
  #    my $skullContext=$skullDoc->attribute('context');
  #    $nb.="#   >$skullName $skullContext.\n";
  #    }
  #  }
  $nb.="#\n";
  return($nb);
  }

#================================================================
# Special Control Methods
#================================================================

# Get authority of current user to perform a specific action
#
#  $action  : Display|Disable|Enable|Edit|Add|Update|Delete
#  $control : 1 - request for control plan, 0 - otherwise
#
# Levels of authority automatically have the authority of the
# lewer levels.
#
#    owner (highest)
#    admin
#    planner
#    guest (lowest)
#
# 
# Permissions required for specific actions
#
#              Non-Control  Control
#              -----------  ------------
#  'Display' - none         none
#  'Disable' - planner      admin
#  'Enable'  - planner      admin
#  'Edit'    - planner      admin
#  'Add'     - planner      admin
#  'Update'  - planner      admin
#  'Delete'  - owner        owner 
#
# New plans are created using 'Add' within a directory.  So one
# must be designated as a planner for a directory to add a new
# plan to it.
#
# 
sub getAuthorized{
  my ($self,$action,$control)=@_;
  return(1) if($action eq 'Display');  # anyone can display
  my $planner=$self->{PLANNER};
  my $userRole=$planner->getUserRole();
  return(1) if($userRole eq 'admin');  # admin can manage all plans
  my $userId=$planner->getUserId();
  my $doc=$self->{DOC};
  my $planDoc=$doc->child('plan');
  my $controlDoc=$planDoc->child('control');
  my $permitsDoc=$controlDoc->child('permits');
  foreach my $permitDoc ($permitsDoc->children('permit')){
    my $permitState=$permitDoc->attribute('state');
    next if($permitState=~/^(disabled|deleted)$/);
    my $permitUser=$permitDoc->attribute('user');
    if($permitUser eq $userId){  
      my $permitRole=$permitDoc->attribute('role');
      $permitRole='planner' if(!$permitRole);
      if($action eq 'delete' || $control){
        return(1) if($permitRole eq 'owner');
        return(0);
        }
      if($control){
        return(1) if($permitRole=~/^(owner|admin)$/);
        return(0);
        }
      return(1) if($permitRole=~/^(owner|admin|planner)$/);
      return(0);
      }
    }
  return(0);
  }

1;
